let _lastRowId=0;

const _RowStatus = {
  NotModified: 0,
  DataModified: 1,
  New: 2,
  NewModified: 3,
  Deleted:4
};

const  _DataHandler= {
  
    /*----------对通过WEBAPI返回的数据进行代理处理，以满足Form和表格对数据行ID和状态的处理*/
    Proxy(data,rowId,fnUpdateModified){
        
        //如果传入的数据已经是Proxy对象，则不处理
        if(data['__dataProxy']==='1'){

            if(fnUpdateModified && data.fnUpdateModified===null){
                data.fnUpdateModified=fnUpdateModified;
            }

            return data;
        }

        return new Proxy(data,
            {
              _rowId:(rowId>0 )? rowId: ( ++_lastRowId ),
              _isSelected:false,
              _fnUpdateModified:fnUpdateModified ,

              get(target,property){

                if(property==='__dataProxy'){
                    return '1' ;
                 }

                // axios和vue框架会读取对象的一些属性，为了避免这些框架读取这些属性时出现失败，所以不再检查属性是否存在
                //调用 JSON.stringify会有toJSON , Vue里面复制 modelValue数据时，不知道为什么会有then的属性
                if(property==='toJSON' || property==='then' || property==='append' || (typeof property)==='symbol' || property.startsWith("__")){
                   return target[property];
                }

                if(property==='resetUpdate'){
                    return ()=>{
                        target['originalData']=null;
                        if(target['rowStatus']!==_RowStatus.New)
                           target['rowStatus']=_RowStatus.NotModified;
                    };
                }

                if(property==='rowId' ){
                    return this._rowId;
                }
                else if(property==="isSelected"){
                   return this._isSelected;
                }
                else if(property==="fnUpdateModified"){
                  return this._fnUpdateModified || null;
                }
                /*
                else if(property==="setDataModified"){
                    return ()=>{

                       if(target['rowStatus']===_RowStatus.NewModified || target['rowStatus']===_RowStatus.DataModified)
                          return;
                      
                        target['rowStatus']=(_RowStatus.New) ? _RowStatus.NewModified : _RowStatus.DataModified;
                    }
                }
                */
                if(!(property in target)){
                    if(property==='rowStatus'){
                       target[property] =  _RowStatus.NotModified;
                    }
                    else if(property==='originalData'){
                       return null;
                    }
                   // else {
                   //     throw new ReferenceError(`属性${property}不存在!`);
                   // }
                }

                 const retValue= target[property];

                 return (retValue===undefined) ? null : retValue;   //如果属性不存在,返回null，方便代码简单只对值是否为 null 进行判断 
              },

              set(target,property,newValue){

                if( (typeof property)==='symbol'){
                  target[property]= newValue;

                  return true;
                }
                else if(property==='__originalData'){
                  target['originalData']= newValue;

                  return true;
                }
                else if(property==="isSelected"){
                   this._isSelected=newValue;

                   return true;
                }
                else if(property==="fnUpdateModified"){
                  this._fnUpdateModified=newValue;

                  return true;
               }


                if( property==='rowId' || property==='originalData' )
                    throw new ReferenceError(`属性${property}是只读的。`);

                // if(property!=='rowStatus' && !(property in target))
                //    throw new ReferenceError(`属性${property}不存在!`);

                  if(target[property] === newValue)   //VUE的控件通过　v-ｍodel 绑定了对象属性，即使没有修改数据，也可能会解发更新model的值，导致数据状态被更新
                      return true ;

                  let original= target['originalData'] || null;
                  let rowStatus  = target['rowStatus'] ||  _RowStatus.NotModified ;

                  if(property==='rowStatus' ){
                       rowStatus =newValue;
                       target['rowStatus'] = newValue;
                       
                      if( rowStatus!== _RowStatus.DataModified || original!==null)
                        return true;
                   } 
                   else{
                        let newStatus = (rowStatus === _RowStatus.NotModified) ? _RowStatus.DataModified :
                                   ( rowStatus === _RowStatus.New) ? _RowStatus.NewModified : rowStatus;

                        if(newStatus!==rowStatus){
                          rowStatus=newStatus;
                          target['rowStatus'] = rowStatus;
                        }
                   }

                   //对任何一属性进行修改，都把当前对象的值赋值到orinialData对象，并修改当前对象的rowStatus的状态
                  if(original === null  && rowStatus === _RowStatus.DataModified && (property in target)  ){
                        original= {};
                        
                        for( let key in target){
                          if( (typeof key)!=='symbol'  && key !=='rowId' && key!=='rowStatus' && key!=='originalData'){
                            original[key] =target[key];
                          }
                      }
                      
                      target['originalData'] = original;
                    }

                    target[property]= newValue;

                    //更新数据表格或表单的修改状态
                    if(this._fnUpdateModified &&  (typeof this._fnUpdateModified)==='function'){
                       this._fnUpdateModified(true);
                    }

                    return true;  //set方法需要返加true,否则会出现  trap returned falsish for property 的错误
              },

              has(target, property){
                if(property==='__dataProxy' || property==='rowId' || property==="isSelected" || 
                   property==='rowStatus' || property==='originalData' || (typeof property)==='symbol' )
                    return true;

                  return (property in target);
              },
            }
        )
    },

    Initialize(srcData,rowId,fnUpdateModified){
        let data={};

        for(let key in srcData){
           if((typeof key)==='symbol'  || key==='rowId' || key==="isSelected" || key==='rowStatus' || key==='originalData'  ) 
              continue;

            const value = srcData[key];
            const valueType = typeof value;

            if(value==='boolean'){
                data[key] = false;
            }
            else{
               data[key]=null;
            }
        }

       let result= _DataHandler.Proxy(data,rowId,fnUpdateModified);

       result['isSelected']= false;
       result['rowStatus'] = _RowStatus.New;

       return result;
    },

    /*----------- 对删除的行，恢复记录原来的数据，并删除 originalData 的数据--------------------*/
    Restore(data){
        let original= data['originalData'] || null;

        data['rowStatus'] = _RowStatus.Deleted;   //先设置状态，后面更新数据的时候才不会去更新 originalData的数据

        for(let key in original){
            if((typeof key)==='symbol' || key==='rowId' || key==="isSelected" ||  key==='rowStatus' || key==='originalData'  ) 
              continue;
            
            data[key] = original[key];
         }

        data['isSelected']=false;
        data['__originalData']=null;  //originalData的代理对象控制了为只读，所以通过特殊的属性来处理

        return data;
    }
}

export { _DataHandler as DataHandler, _RowStatus as RowStatus }

