import dayjs from 'dayjs';

export function NumberFormat(value, format){
     
     const formatType = typeof format;
     let locale='zh-Hans-CN';
     let style='decimal';
     let currency='CNY';
     let currencyDisplay='symbol';
     let useGrouping=true;
     let maximumFractionDigits =2;
     let minimumFractionDigits =2; 
    
     if(format===null || format==='')
        return value;

     if(typeof value==='string'){
        value = Number.parseFloat(value);
        if(Number.isNaN(value))
            return 'NaN';
     }

     if(formatType ==='object'){
       const formater = new Intl.NumberFormat(locale, format );

       return formater.format(value || 0 );
     }

     if(formatType ==='string'){
        //如果是货币格式，在前面加货币代码
        let index= format.indexOf(":");

        if(index>0){
            currency = format.substring(0,index).toUpperCase().trim();
            format = format.substring(index+1);
            style ='currency'
        }
        else if(format.endsWith("%")){
            style="percent";
            format =format.substring(0,format.length -1);
        }
     }

     if(format.indexOf(",")<=0){
        useGrouping=false;
     }

     if(currency ==='RMB' || currency==='￥'){
        currency='CNY';
     }
     else if(currency==='$'){
        currency='USD'
     }

     if(format==='#,##0' || format ==='#,###' ){

        if( format ==='#,###'  && !value )
            return "";

        maximumFractionDigits=0;
        minimumFractionDigits =0;
     }
     
     else if(format==='0'){
        maximumFractionDigits=0;
        minimumFractionDigits =0;
     }
   
    const  formater =new Intl.NumberFormat(locale,{style, currency, currencyDisplay , useGrouping ,  maximumFractionDigits,minimumFractionDigits } ); 

    return formater.format(value || 0 );
}


export function DateTimeFormat(value, format){
    if(value===null || value===undefined || value==='')
        return "";

    return dayjs(value).format(format);
}
